import React from 'react';
import PropTypes from 'prop-types';
import { GraphQLSchema } from 'graphql';
import MD from 'markdown-it';
import { normalizeWhitespace } from '../utility/normalizeWhitespace';
import onHasCompletion from '../utility/onHasCompletion';
import commonKeys from '../utility/commonKeys';
const md = new MD();
const AUTO_COMPLETE_AFTER_KEY = /^[a-zA-Z0-9_@(]$/;
export class QueryEditor extends React.Component {
    constructor(props) {
        super();
        this._onKeyUp = (cm, event) => {
            if (AUTO_COMPLETE_AFTER_KEY.test(event.key)) {
                this.editor.execCommand('autocomplete');
            }
        };
        this._onEdit = () => {
            if (!this.ignoreChangeEvent) {
                this.cachedValue = this.editor.getValue();
                if (this.props.onEdit) {
                    this.props.onEdit(this.cachedValue);
                }
            }
        };
        this._onHasCompletion = (cm, data) => {
            onHasCompletion(cm, data, this.props.onHintInformationRender);
        };
        this.cachedValue = props.value || '';
    }
    componentDidMount() {
        const CodeMirror = require('codemirror');
        require('codemirror/addon/hint/show-hint');
        require('codemirror/addon/comment/comment');
        require('codemirror/addon/edit/matchbrackets');
        require('codemirror/addon/edit/closebrackets');
        require('codemirror/addon/fold/foldgutter');
        require('codemirror/addon/fold/brace-fold');
        require('codemirror/addon/search/search');
        require('codemirror/addon/search/searchcursor');
        require('codemirror/addon/search/jump-to-line');
        require('codemirror/addon/dialog/dialog');
        require('codemirror/addon/lint/lint');
        require('codemirror/keymap/sublime');
        require('codemirror-graphql/hint');
        require('codemirror-graphql/lint');
        require('codemirror-graphql/info');
        require('codemirror-graphql/jump');
        require('codemirror-graphql/mode');
        this.editor = CodeMirror(this._node, {
            value: this.props.value || '',
            lineNumbers: true,
            tabSize: 2,
            mode: 'graphql',
            theme: this.props.editorTheme || 'graphiql',
            keyMap: 'sublime',
            autoCloseBrackets: true,
            matchBrackets: true,
            showCursorWhenSelecting: true,
            readOnly: this.props.readOnly ? 'nocursor' : false,
            foldGutter: {
                minFoldSize: 4,
            },
            lint: {
                schema: this.props.schema,
            },
            hintOptions: {
                schema: this.props.schema,
                closeOnUnfocus: false,
                completeSingle: false,
                container: this._node,
            },
            info: {
                schema: this.props.schema,
                renderDescription: text => md.render(text),
                onClick: reference => this.props.onClickReference(reference),
            },
            jump: {
                schema: this.props.schema,
                onClick: reference => this.props.onClickReference(reference),
            },
            gutters: ['CodeMirror-linenumbers', 'CodeMirror-foldgutter'],
            extraKeys: {
                'Cmd-Space': () => this.editor.showHint({ completeSingle: true, container: this._node }),
                'Ctrl-Space': () => this.editor.showHint({ completeSingle: true, container: this._node }),
                'Alt-Space': () => this.editor.showHint({ completeSingle: true, container: this._node }),
                'Shift-Space': () => this.editor.showHint({ completeSingle: true, container: this._node }),
                'Shift-Alt-Space': () => this.editor.showHint({ completeSingle: true, container: this._node }),
                'Cmd-Enter': () => {
                    if (this.props.onRunQuery) {
                        this.props.onRunQuery();
                    }
                },
                'Ctrl-Enter': () => {
                    if (this.props.onRunQuery) {
                        this.props.onRunQuery();
                    }
                },
                'Shift-Ctrl-C': () => {
                    if (this.props.onCopyQuery) {
                        this.props.onCopyQuery();
                    }
                },
                'Shift-Ctrl-P': () => {
                    if (this.props.onPrettifyQuery) {
                        this.props.onPrettifyQuery();
                    }
                },
                'Shift-Ctrl-F': () => {
                    if (this.props.onPrettifyQuery) {
                        this.props.onPrettifyQuery();
                    }
                },
                'Shift-Ctrl-M': () => {
                    if (this.props.onMergeQuery) {
                        this.props.onMergeQuery();
                    }
                },
                ...commonKeys,
                'Cmd-S': () => {
                    if (this.props.onRunQuery) {
                    }
                },
                'Ctrl-S': () => {
                    if (this.props.onRunQuery) {
                    }
                },
            },
        });
        this.editor.on('change', this._onEdit);
        this.editor.on('keyup', this._onKeyUp);
        this.editor.on('hasCompletion', this._onHasCompletion);
        this.editor.on('beforeChange', this._onBeforeChange);
    }
    componentDidUpdate(prevProps) {
        const CodeMirror = require('codemirror');
        this.ignoreChangeEvent = true;
        if (this.props.schema !== prevProps.schema) {
            this.editor.options.lint.schema = this.props.schema;
            this.editor.options.hintOptions.schema = this.props.schema;
            this.editor.options.info.schema = this.props.schema;
            this.editor.options.jump.schema = this.props.schema;
            CodeMirror.signal(this.editor, 'change', this.editor);
        }
        if (this.props.value !== prevProps.value &&
            this.props.value !== this.cachedValue) {
            this.cachedValue = this.props.value;
            this.editor.setValue(this.props.value);
        }
        this.ignoreChangeEvent = false;
    }
    componentWillUnmount() {
        if (this.editor) {
            this.editor.off('change', this._onEdit);
            this.editor.off('keyup', this._onKeyUp);
            this.editor.off('hasCompletion', this._onHasCompletion);
            this.editor = null;
        }
    }
    render() {
        return (React.createElement("section", { className: "query-editor", "aria-label": "Query Editor", ref: node => {
                this._node = node;
            } }));
    }
    getCodeMirror() {
        return this.editor;
    }
    getClientHeight() {
        return this._node && this._node.clientHeight;
    }
    _onBeforeChange(instance, change) {
        if (change.origin === 'paste') {
            const text = change.text.map(normalizeWhitespace);
            change.update(change.from, change.to, text);
        }
    }
}
QueryEditor.propTypes = {
    schema: PropTypes.instanceOf(GraphQLSchema),
    value: PropTypes.string,
    onEdit: PropTypes.func,
    readOnly: PropTypes.bool,
    onHintInformationRender: PropTypes.func,
    onClickReference: PropTypes.func,
    onCopyQuery: PropTypes.func,
    onPrettifyQuery: PropTypes.func,
    onMergeQuery: PropTypes.func,
    onRunQuery: PropTypes.func,
    editorTheme: PropTypes.string,
};
//# sourceMappingURL=QueryEditor.js.map